@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -284,16 +284,16 @@ char *xdebug_raw_url_encode(char const *s, int len, int *new_length, int skip_sl
 char *xdebug_path_from_url(const char *fileurl TSRMLS_DC)
 {
 	/* deal with file: url's */
-	char dfp[PATH_MAX * 2];
-	const char *fp = dfp, *efp = fileurl;
+	char *dfp = NULL;
+	const char *fp = NULL, *efp = fileurl;
 #ifdef PHP_WIN32
 	int l = 0;
 	int i;
 #endif
-	char *tmp = NULL, *ret = NULL;;
+	char *tmp = NULL, *ret = NULL;
 
-	memset(dfp, 0, sizeof(dfp));
-	strncpy(dfp, efp, sizeof(dfp) - 1);
+	dfp = xdstrdup(efp);
+	fp = dfp;
 	xdebug_raw_url_decode(dfp, strlen(dfp));
 	tmp = strstr(fp, "file://");
 
@@ -316,15 +316,10 @@ char *xdebug_path_from_url(const char *fileurl TSRMLS_DC)
 		ret = xdstrdup(fileurl);
 	}
 
+	free(dfp);
 	return ret;
 }
 
-#if PHP_VERSION_ID >= 50400
-# define VIRTUAL_FILE_EX_TSRMLS_CC TSRMLS_CC
-#else
-# define VIRTUAL_FILE_EX_TSRMLS_CC
-#endif
-
 /* fake URI's per IETF RFC 1738 and 2396 format */
 char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 {
@@ -352,7 +347,7 @@ char *xdebug_path_to_url(const char *fileurl TSRMLS_DC)
 		new_state.cwd = strdup(cwd);
 		new_state.cwd_length = strlen(cwd);
 
-		if (!virtual_file_ex(&new_state, fileurl, NULL, 1 VIRTUAL_FILE_EX_TSRMLS_CC)) {
+		if (!virtual_file_ex(&new_state, fileurl, NULL, 1 TSRMLS_CC)) {
 			char *s = estrndup(new_state.cwd, new_state.cwd_length);
 			tmp = xdebug_sprintf("file://%s",s);
 			efree(s);
@@ -668,6 +663,40 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 	return fname.l;
 }
 
+int xdebug_format_file_link(char **filename, const char *error_filename, int error_lineno TSRMLS_DC)
+{
+	xdebug_str fname = {0, 0, NULL};
+	char      *format = XG(file_link_format);
+
+	while (*format)
+	{
+		if (*format != '%') {
+			xdebug_str_addl(&fname, (char *) format, 1, 0);
+		} else {
+			format++;
+			switch (*format)
+			{
+				case 'f': /* filename */
+					xdebug_str_add(&fname, xdebug_sprintf("%s", error_filename), 1);
+					break;
+
+				case 'l': /* line number */
+					xdebug_str_add(&fname, xdebug_sprintf("%d", error_lineno), 1);
+					break;
+
+				case '%': /* literal % */
+					xdebug_str_addl(&fname, "%", 1, 0);
+					break;
+			}
+		}
+		format++;
+	}
+
+	*filename = fname.d;
+
+	return fname.l;
+}
+
 void xdebug_open_log(TSRMLS_D)
 {
 	/* initialize remote log file */
@@ -680,6 +709,8 @@ void xdebug_open_log(TSRMLS_D)
 		fprintf(XG(remote_log_file), "Log opened at %s\n", timestr);
 		fflush(XG(remote_log_file));
 		xdfree(timestr);
+	} else if (strlen(XG(remote_log))) {
+		php_log_err(xdebug_sprintf("XDebug could not open the remote debug file '%s'.", XG(remote_log)) TSRMLS_CC);
 	}
 }
 
